/**
 * @memberof module:apeman-app
 * @function runApp
 * @param {Apemanfile} apemanfile
 * @param {string[]} names
 * @param {object} options
 * @param {function} options.setup
 * @returns Promise
 */

'use strict'

const argx = require('argx')
const path = require('path')
const assert = require('assert')
const RunLogger = require('../logging/run_logger')
const getApps = require('./get_apps')
const Promise = require('apemanpromise')
const appServer = require('./app_server')
const appEndpoints = require('./app_endpoints')

/** @lends runApp */
function runApp (configuration, names, options = {}) {
  let args = argx(arguments)
  configuration = args.shift()
  names = [].concat(args.shift('string|array'))
  options = args.pop('object') || {}

  let port = options.port || 3000
  let setup = options.setup

  assert.ok(configuration, 'apemanfile is required.')

  let logger = new RunLogger({
    file: options.log && path.resolve(options.log)
  })

  // Increase max listeners to avoid leak warning.
  process.setMaxListeners(
    process.getMaxListeners() + 1
  )

  process.on('uncaughtException', (err) => {
    logger.logUncaughtException(err)
  })


  logger.logAppStarting(names)
  let apps = getApps(configuration, names)
  let endpoints = appEndpoints(apps)
  let server = appServer(endpoints)
  server = _setup(server, setup)
  process.emit('apeman:app:start')
  if (process.send) {
    process.send('apeman:app:start')
  }
  server.listen(port)
  server.close = ((close) => function closeWrap () {
    if (server.closed) {
      return
    }
    server.closed = true
    logger.logAppStopping(names)
    return new Promise((resolve, reject) => {
      close.call(server, (err) => {
        process.nextTick(() => {
          process.emit('apeman:app:stop')
          if (process.send) {
            process.send('apeman:app:stop')
          }

          if (err) {
            reject(err)
          } else {
            logger.logAppStopped(port)
            resolve()
          }
        })
      })
    })
  })(server.close)
  logger.logAppStarted(port)
  return Promise.resolve(server)
}

function _setup (server, setup) {
  if (!setup) {
    return server
  }
  if (typeof setup === 'string') {
    setup = require(setup)
  }

  server = setup(server)
  if (!server) {
    throw new Error('setup should return the server instance!')
  }
  return server
}

module.exports = runApp
