/**
 * Logger for list.
 * @memberof module:apeman-app
 * @constructor ListLogger
 * @returns A logger instance.
 */

'use strict'

const colorprint = require('colorprint')
const path = require('path')
const textconv = require('textconv')
const arraysort = require('arraysort')
const colors = colorprint.colors

const NAME_MIN_LENGTH = 24
const DESC_MAX_LENGTH = 64

/** @lends ListLogger */
module.exports = colorprint.define(
  /** @lends ListLogger.prototype */
  {
    /**
     * Log app list.
     * @param {object} apps - Apps to show.
     * @param {object[]} childApps - Child apps to  show.
     */
    logList (apps, childApps) {
      const s = this
      s.debug('')
      s.debug('  Available apps:')
      let appNames = Object.keys(apps)
      if (appNames.length > 0) {
        s.debug('')
        appNames.forEach((name) =>
          s._logApp(name, s._descApp(apps[ name ]))
        )
      }
      s.debug('')
      for (let data of (childApps || [])) {
        let { cwd, namespace } = data
        cwd = (cwd && path.relative(process.cwd(), cwd)) || null
        let location = (cwd ? '"./' + cwd + '"' : '')
        s.debug(colors.black.italic('    /* Apps in ' + (location || namespace) + ' */'))
        Object.keys(data.apps)
          .sort(arraysort.stringCompare())
          .filter((name) => {
            let apps = data.apps[ name ]
            return Object.keys(apps || {}).length > 0
          })
          .forEach((name) => {
            let app = data.apps[ name ]
            let fullName = [ namespace, name ].join('#')
            s._logApp(fullName, s._descApp(app))
          })
        s.debug('')
      }
      s.debug('')
      s.debug('')
    },
    _logApp (name, desc) {
      const s = this
      desc = textconv.chop(desc, DESC_MAX_LENGTH, { ellipsis: true })
      let comment = desc ? colors.blackBright([ '//', desc ].join(' ')) : ''
      s.debug('    %s%s', colors.black.bold(textconv.pad(name, NAME_MIN_LENGTH)), comment)
    },
    _descApp (app) {
      const s = this
      if (Array.isArray(app)) {
        return app
          .map((name) =>
            s._descApp(name) || ('`' + String(name) + '`')
          )
          .map((desc) =>
            textconv.strip(desc.trim(), /\./)
          ).join(', ')
      }
      let $desc = app.$desc || app.$description
      if ($desc) {
        return $desc
      }
      if (typeof app === 'function') {
        return 'function() {/* ... */}'
      }
      if (app[ '/' ]) {
        let rootDesc = s._descApp(app[ '/' ])
        if (rootDesc) {
          return rootDesc
        }
      }
      return app.desc || ''
    }
  }
)
