/**
 * Logger for run.
 * @memberof module:apeman-app
 * @constructor RunLogger
 * @returns A logger instance.
 */

'use strict'

const colorprint = require('colorprint')
const fileStreamRotator = require('file-stream-rotator')
const mkdirp = require('mkdirp')
const path = require('path')
const {EOL} = require('os')

/** @lends RunLogger */
module.exports = colorprint.define(
  /** @lends RunLogger.prototype */
  {
    rotate: 'daily',
    dateFormat: 'YYYYMMDD',
    _streams: null,
    _getStream (filename) {
      const s = this
      let streams = s._streams
      if (!streams) {
        streams = s._streams = {}
      }
      let stream = streams[ filename ]
      if (!stream) {
        mkdirp.sync(path.dirname(filename))
        stream = streams[ filename ] = fileStreamRotator.getStream({
          filename: filename,
          frequency: s.rotate,
          date_format: s.dateFormat
        })
      }
      return stream
    },
    _getFilename (namespace) {
      const s = this
      let filename = s.file
      if (!filename) {
        return null
      }
      let dirname = path.dirname(filename)
      let extname = path.extname(filename)
      let basename = path.basename(filename, extname)
      return path.join(dirname, `${basename}#${namespace}-%DATE%${extname}`)
    },
    _writeToFile (msg, namespace) {
      const s = this
      let filename = s._getFilename(namespace)
      if (!filename) {
        return
      }
      let stream = s._getStream(filename)
      let now = new Date().toLocaleString()
      stream.write(`[${now}] ${String(msg)}`)
      stream.write(EOL)
    },
    logAppStarting (name) {
      const s = this
      let msg = `Starting app "${[].concat(name).join(',')}"...`
      s.notice(msg)
      s._writeToFile(msg, 'process')
    },
    logAppStarted (port) {
      const s = this
      let msg = `\tApp started at port: ${port}`
      s.notice(msg)
      s._writeToFile(msg, 'process')
    },
    logAppStopping (name) {
      const s = this
      let msg = `Stopping app "${[].concat(name).join(',')}"...`
      s.notice(msg)
      s._writeToFile(msg, 'process')
    },
    logAppStopped (port) {
      const s = this
      let msg = `\tApp stopped at port: ${port}`
      s.notice(msg)
      s._writeToFile(msg, 'process')
    },
    logAppStartFailed (err) {
      const s = this
      s.error(err.stack || err)
      s._writeToFile(err.trace || err.message || err, 'error')
    },
    logUncaughtException (err) {
      const s = this
      s.error(err.stack || err)
      s._writeToFile(err.stack || err.message || err, 'error')
    },
    logUnhandledError (err) {
      const s = this
      s.error(err.stack || err)
      s._writeToFile(err.stack || err.message || err, 'error')
    }
  }
)
